<?php

namespace App\Http\Controllers;

use App\Constants\Status;
use App\Lib\CoinPaymentHosted;
use App\Lib\CurlRequest;
use App\Models\Account;
use App\Models\AdminNotification;
use App\Models\Deposit;
use App\Models\Gateway;
use App\Models\ReferralLog;
use App\Models\Track;
use App\Models\Withdrawal;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

class PaymentController extends Controller
{
    public function depositWallet(Request $request)
    {
        $request->validate([
            'amount' => 'required|numeric|gt:0'
        ]);
        $account = session('ACCOUNT');
        $account = Account::where('wallet', $account)->first();

        $gateway     = Gateway::first();

        $trx = getTrx();

        $response       = CurlRequest::curlPostContent('https://api.nowpayments.io/v1/payment', json_encode([
            'price_amount'     => $request->amount,
            'price_currency'   => gs('cur_text'),
            'pay_currency'     => gs('cur_text'),
            'ipn_callback_url' => route('ipn.nowPayment.deposit'),
            'order_id'         => $trx,

        ]), [
            "x-api-key: $gateway->key_1",
            'Content-Type: application/json'
        ]);
        $response = json_decode($response);

        if (!$response) {
            $data['error'] = 'Some problem occurred with api';
            return response()->json($data);
        }
        if (@$response->status === false) {
            $data['error'] = $response->message;
            return response()->json($data);
        }

        $deposit             = new Deposit();
        $deposit->account_id = $account->id;
        $deposit->amount     = $request->amount;
        $deposit->wallet     = $account->wallet;
        $deposit->trx        = $trx;
        $deposit->status     = 0;
        $deposit->save();

        $data['amount']   = $response->pay_amount;
        $data['sendto']   = $response->pay_address;
        $data['img']      = cryptoQR($response->pay_address);
        $data['currency'] = $response->pay_currency;

        return response()->json($data);
    }

    public function ipnDeposit()
    {

        if (isset($_SERVER['HTTP_X_NOWPAYMENTS_SIG']) && !empty($_SERVER['HTTP_X_NOWPAYMENTS_SIG'])) {
            $recived_hmac = $_SERVER['HTTP_X_NOWPAYMENTS_SIG'];
            $request_json = file_get_contents('php://input');
            $request_data = json_decode($request_json, true);
            ksort($request_data);
            $sorted_request_json = json_encode($request_data, JSON_UNESCAPED_SLASHES);
            if ($request_json !== false && !empty($request_json)) {
                $gateway    = Gateway::where('alias', 'NowPaymentsHosted')->first();
                $gatewayAcc = json_decode($gateway->gateway_parameters);
                $hmac       = hash_hmac("sha512", $sorted_request_json, trim($gatewayAcc->secret_key->value));
                if ($hmac == $recived_hmac) {
                    if ($request_data['payment_status'] == 'confirmed' || $request_data['payment_status'] == 'finished') {
                        if ($request_data['actually_paid'] == $request_data['pay_amount']) {
                            $deposit = Deposit::initiated()->where('trx', $request_data['order_id'])->first();
                            if ($deposit) {

                                $deposit->amount = @$request_data['amount'] ? $request_data['amount'] : $deposit->amount;
                                $deposit->status = 1;
                                $deposit->save();
                                $general           = gs();
                                $track             = Track::where('account_id', $deposit->account_id)->first();
                                $track->account_id = $deposit->account_id;
                                $track->speed += intval($deposit->amount / $general->dhs_price);
                                $track->save();

                                $account = Account::find($deposit->account_id);
                                $adminNotification = new AdminNotification();
                                $adminNotification->account_id = $account->id;
                                $adminNotification->title = 'Deposit submited from '.$account->wallet;
                                $adminNotification->click_url = route('admin.deposit.index');
                                $adminNotification->save();

                                if ($account && $general->referral_system) {
                                    $track = Track::where('account_id', $account->ref_by)->where('status', Status::ACTIVE)->first();

                                    if ($track && $general->referral_bonus > 0) {
                                        $bonusAmount = ($deposit->amount * $general->referral_bonus) / 100;
                                        $track->balance += floatval($bonusAmount);
                                        $track->save();

                                        if ($track) {
                                            $refLog                 = new ReferralLog();
                                            $refLog->account_id     = $account->ref_by;
                                            $refLog->ref_user_id    = $deposit->account_id;
                                            $refLog->deposit_amount = $deposit->amount;
                                            $refLog->bonus_value    = floatval($bonusAmount);
                                            $refLog->save();
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    public function withdrawMoney(Request $request)
    {
        $account = session()->get('ACCOUNT');
        $account = Account::where('wallet', $account)->first();
        $track   = Track::where('account_id', $account->id)->first();


        if ($track) {
            $gateway  = Gateway::first();
            $validate = Validator::make($request->except('_token'), [
                "amount" => "required|numeric|min:$gateway->minimum_withdraw|max:$gateway->maximum_withdraw",
            ]);

            if ($validate->fails()) {
                return response()->json(['error' => $validate->errors()->all()]);
            }
            if ($request->amount > $track->balance) {
                return response()->json(['error' => 'You don\'t have sufficient balance']);
            }

            $accountCheck = $this->accountCheck($account->wallet, $gateway->key_1);
            if (!$accountCheck) {
                return response()->json(['error' => 'No account found']);
            }

            $auth = CurlRequest::curlPostContent('https://api.nowpayments.io/v1/auth', json_encode([
                'email' => @$gateway->email,
                'password' => @$gateway->password,
            ]), [
                'Content-Type: application/json',
            ]);

            $authToken = json_decode($auth);

            if(@$authToken->error){

                $adminNotification = new AdminNotification();
                $adminNotification->account_id = $account->id;
                $adminNotification->title = 'Wrong gateway credentials';
                $adminNotification->click_url = "#";
                $adminNotification->save();

                return response()->json(['error' => 'Something went wrong gateway login']);
            }
            if(@$authToken->status == false){

                $adminNotification = new AdminNotification();
                $adminNotification->account_id = $account->id;
                $adminNotification->title = 'Wrong gateway credentials';
                $adminNotification->click_url = "#";
                $adminNotification->save();

                return response()->json(['error' => 'Something went wrong gateway login']);
            }

            $body = json_encode([
                "ipn_callback_url" => "https://nowpayments.io",
                "withdrawals" => [
                    [
                        'address' => $track->account->wallet,
                        'currency' =>
                        strtolower(gs('cur_text')),
                        'amount' => $request->amount,
                        "ipn_callback_url" =>
                        "https://nowpayments.io",
                    ],
                ],
            ]);

            $curl = curl_init();
            curl_setopt_array($curl, array(
                CURLOPT_URL => 'https://api.nowpayments.io/v1/payout',
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'POST',
                CURLOPT_POSTFIELDS => $body,
                CURLOPT_HTTPHEADER => array(
                    "Authorization: Bearer $authToken->token",
                    "x-api-key: $gateway->key_1",
                    'Content-Type: application/json'
                ),
            ));

            $response = curl_exec($curl);
            curl_close($curl);
            $data = json_decode($response);

            if ($data && @$data->status === false) {
                $track->balance  = $track->balance + $request->amount;
                $track->withdraw = $track->withdraw - $request->amount;
                $track->save();
                return response()->json(['error' => 'Something went wrong! ' . @$data['error']]);
            } else {
                $track->balance  = $track->balance - $request->amount;
                $track->withdraw = $track->withdraw + $request->amount;
                $track->save();

                $withdrawal             = new Withdrawal();
                $withdrawal->account_id = $account->id;
                $withdrawal->amount     = $request->amount;
                $withdrawal->trx        = getTrx();
                $withdrawal->save();

                $adminNotification = new AdminNotification();
                $adminNotification->account_id = $account->id;
                $adminNotification->title = 'Withdraw from '.$track->account->wallet;
                $adminNotification->click_url = route('admin.withdraw.index');
                $adminNotification->save();
                
            }
        } else {
            return response()->json(['error' => 'Track not found']);
        }
    }


    private function accountCheck($wallet, $key)
    {
        $curl = curl_init();

        curl_setopt_array($curl, array(
        CURLOPT_URL => 'https://api.nowpayments.io/v1/payout/validate-address',
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => '',
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => 'POST',
        CURLOPT_POSTFIELDS =>'{
            "address": "'.$wallet.'",
            "currency": "'.strtolower(gs('cur_text')).'",
            "extra_id":null
        }',
        CURLOPT_HTTPHEADER => array(
            'x-api-key: '.$key,
            'Content-Type: application/json'
        ),
        ));

        $response = curl_exec($curl);

        curl_close($curl);
        if($response == 'OK'){
            return true;
        }
        return false;

    }
}
